-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 43 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=43;

-- Modifications for a config container and for host configs container

--
-- Create a new table to store config containers
--
create table CONFIG_CONTAINERS (
       CONFIG_CONTAINER_ID bigint not null,
       OPTIMISTIC_LOCK_VERSION bigint not null,
       CONFIG_TYPE varchar(255) not null unique,
       CONFIG_REVISION_ID bigint,
       primary key (CONFIG_CONTAINER_ID)
) engine=InnoDB;

--
-- Create a new column in CONFIG_REVISIONS to store config container id
--
alter table CONFIG_REVISIONS
      add column CONFIG_CONTAINER_ID bigint;

--
-- Create a new column in HOSTS to store config container id
--
alter table HOSTS
      add column CONFIG_CONTAINER_ID bigint;

--
-- Create a new column in CONFIGS to store config container id
--
alter table CONFIGS
      add column CONFIG_CONTAINER_ID bigint;

--
-- Add a column in the AUDITS table to refer to a config container to
-- facilitate auditing of updates to configs 'own'-ed by the config container.
--
alter table AUDITS
      add column CONFIG_CONTAINER_ID bigint;

--
-- Drop obsolete foreign key constraints
--
alter table CONFIG_REVISIONS
      drop foreign key FK_CONFIG_REVISION_HOST;

alter table HOSTS
      drop foreign key FK_HOST_CONFIG_REVISION;

--
-- Clean up unwanted columns from the config revision table.
-- Remove existing config revisions that refer to hosts since we no
--   longer have config revisions associated with hosts.
-- Drop the host id column
--
delete from CONFIG_REVISIONS where HOST_ID is not null;
alter table CONFIG_REVISIONS
      drop column HOST_ID;

--
-- Clean up unwanted columns from the hosts table.
-- Drop the config revision id column
--
alter table HOSTS
      drop column CONFIG_REVISION_ID;

--
-- Insert a config container to track host configs.
--
insert into CONFIG_CONTAINERS (CONFIG_CONTAINER_ID, OPTIMISTIC_LOCK_VERSION, CONFIG_TYPE)
       values (1, 1, 'ALL_HOSTS');

--
-- Insert a base revision that is visible for each config container
-- Automatic incrementing of the revision ID is temporarily enabled to make it
-- easier to create a new revision.
-- After inserting revisions for each host, auto incrementing is turned off.

alter table SERVICES
  drop foreign key FK_SERVICE_CONFIG_REVISION;

alter table CONFIGS
  drop foreign key FK_CONFIG_CONFIG_REVISION;

alter table CONFIG_REVISIONS
      modify column REVISION_ID bigint not null auto_increment;

insert into CONFIG_REVISIONS (CONFIG_CONTAINER_ID, CREATED_INSTANT, MESSAGE, VISIBLE)
      select CONFIG_CONTAINER_ID, unix_timestamp(now()) * 1000, 'Base revision', 1
      from CONFIG_CONTAINERS cc;

alter table CONFIG_REVISIONS
      modify column REVISION_ID bigint not null;

alter table SERVICES
  add constraint FK_SERVICE_CONFIG_REVISION
  foreign key (CONFIG_REVISION_ID)
  references CONFIG_REVISIONS (REVISION_ID);

alter table CONFIGS
  add constraint FK_CONFIG_CONFIG_REVISION
  foreign key (REVISION_ID)
  references CONFIG_REVISIONS (REVISION_ID);

--
-- Link each config container to its current config revision.
--
update CONFIG_CONTAINERS cc
      set CONFIG_REVISION_ID=
      (select REVISION_ID
      from CONFIG_REVISIONS r
      where cc.CONFIG_CONTAINER_ID=r.CONFIG_CONTAINER_ID);

--
-- Link each host to the hosts config container
--
update HOSTS h
      set CONFIG_CONTAINER_ID=
      (select CONFIG_CONTAINER_ID
      from CONFIG_CONTAINERS cc
      where cc.CONFIG_TYPE='ALL_HOSTS');

--
-- All data insertion is done. Set up, update indices and constraints.
--
alter table CONFIGS
      add index IDX_CONFIG_CONFIG_CONTAINER (CONFIG_CONTAINER_ID),
      add constraint FK_CONFIG_CONFIG_CONTAINER
      foreign key (CONFIG_CONTAINER_ID)
      references CONFIG_CONTAINERS (CONFIG_CONTAINER_ID);

alter table CONFIG_REVISIONS
      add index IDX_CONFIG_REVISIONS_CONFIG_CONTAINER (CONFIG_CONTAINER_ID),
      add constraint FK_CONFIG_REVISION_CONFIG_CONTAINER
      foreign key (CONFIG_CONTAINER_ID)
      references CONFIG_CONTAINERS (CONFIG_CONTAINER_ID);

alter table CONFIG_CONTAINERS
      add index IDX_CONFIG_CONTAINER_CONFIG_REVISION (CONFIG_REVISION_ID),
      add constraint FK_CONFIG_CONTAINER_CONFIG_REVISION
      foreign key (CONFIG_REVISION_ID)
      references CONFIG_REVISIONS (REVISION_ID);

alter table HOSTS
      add index IDX_HOST_CONFIG_CONTAINER (CONFIG_CONTAINER_ID),
      add constraint FK_HOST_CONFIG_CONTAINER
      foreign key (CONFIG_CONTAINER_ID)
      references CONFIG_CONTAINERS (CONFIG_CONTAINER_ID);
